import controlP5.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentAdapter;
import java.awt.Insets;
import java.awt.Rectangle;

IFS redIFS;
IFS greenIFS;
IFS blueIFS;

PImage compo;

Hist3d hist3d;

float[][] redAdd = new float[][] { {0,0}, {0,0} };
float[][] greenAdd = new float[][] { {0,0}, {0,0} };
float[][] blueAdd = new float[][] { {0,0}, {0,0} };

float[][] redMul = new float[][] { {1,1}, {1,1} };
float[][] greenMul = new float[][] { {1,1}, {1,1} };
float[][] blueMul = new float[][] { {1,1}, {1,1} };

ControlP5 controlP5;
ControlWindow controlWindow;
Slider ra11, ra12, ra21, ra22, rb11, rb12, rb21, rb22;
Slider ga11, ga12, ga21, ga22, gb11, gb12, gb21, gb22;
Slider ba11, ba12, ba21, ba22, bb11, bb12, bb21, bb22;

void setup() {
  colorMode(RGB,1);

  hist3d = new Hist3d();

  size(512,512);
  init(512,512);
  frame.setResizable(true);
  frame.addComponentListener(new ComponentAdapter() { 
   public void componentResized(ComponentEvent e) { 
     if(e.getSource()==frame) {
       Rectangle bounds = frame.getBounds();
       Insets insets = frame.getInsets();
       int w = bounds.width - insets.left - insets.right;
       int h = bounds.height - insets.top - insets.bottom;
       if((w % 2) != 0 || (h % 2) != 0) {
         frame.resize(bounds.width - (w % 2), bounds.height - (h % 2));
       }
       println("resize frame : " + w + " x " + h);
       init(w, h);
     } 
   } 
  });

  /* User Interface */
  controlP5 = new ControlP5(this);  
  PFont p = createFont("Arial",10); 
  controlP5.setControlFont(p,10);

  controlWindow = controlP5.addControlWindow("controlP5window",50,50,400,300);
  controlWindow.hideCoordinates();
  controlWindow.setBackground(color(40));
  controlWindow.setTitle("IFS Parameters");  

  ra11 = controlP5.addSlider("RA11",-5,5);
  ra12 = controlP5.addSlider("RA12",-5,5); ra12.linebreak();
  ra21 = controlP5.addSlider("RA21",-5,5);
  ra22 = controlP5.addSlider("RA22",-5,5); ra22.linebreak();
  rb11 = controlP5.addSlider("RB11",-1,1);
  rb12 = controlP5.addSlider("RB12",-1,1); rb12.linebreak();
  rb21 = controlP5.addSlider("RB21",-1,1);
  rb22 = controlP5.addSlider("RB22",-1,1); rb22.linebreak();

  ga11 = controlP5.addSlider("GA11",-5,5);
  ga12 = controlP5.addSlider("GA12",-5,5); ga12.linebreak();
  ga21 = controlP5.addSlider("GA21",-5,5);
  ga22 = controlP5.addSlider("GA22",-5,5); ga22.linebreak();
  gb11 = controlP5.addSlider("GB11",-1,1);
  gb12 = controlP5.addSlider("GB12",-1,1); gb12.linebreak();
  gb21 = controlP5.addSlider("GB21",-1,1);
  gb22 = controlP5.addSlider("GB22",-1,1); gb22.linebreak();

  ba11 = controlP5.addSlider("BA11",-5,5);
  ba12 = controlP5.addSlider("BA12",-5,5); ba12.linebreak();
  ba21 = controlP5.addSlider("BA21",-5,5);
  ba22 = controlP5.addSlider("BA22",-5,5); ba22.linebreak();
  bb11 = controlP5.addSlider("BB11",-1,1);
  bb12 = controlP5.addSlider("BB12",-1,1); bb12.linebreak();
  bb21 = controlP5.addSlider("BB21",-1,1);
  bb22 = controlP5.addSlider("BB22",-1,1); bb22.linebreak();

  for(Controller s :
    new Controller[] {ra11, ra12, ra21, ra22, rb11, rb12, rb21, rb22,
      ga11, ga12, ga21, ga22, gb11, gb12, gb21, gb22,
      ba11, ba12, ba21, ba22, bb11, bb12, bb21, bb22}) {
        s.moveTo(controlWindow);
      }

  controlP5.addButton("randomize").moveTo(controlWindow);
  controlP5.addButton("empty").moveTo(controlWindow);
  controlP5.addButton("export").moveTo(controlWindow);
  controlP5.addButton("rotate").moveTo(controlWindow);
  
}

void init(int w, int h) {
  redIFS = new IFS(w, h);
  redIFS.setParameters( redAdd, redMul);
  greenIFS = new IFS(w, h);
  greenIFS.setParameters( greenAdd, greenMul);
  blueIFS = new IFS(w, h);
  blueIFS.setParameters( blueAdd, blueMul);
}

void draw() {
  background(0);

  redIFS.iterate();
  greenIFS.iterate();
  blueIFS.iterate();

  compo = compose(
    redIFS.getImage(),
    greenIFS.getImage(),
    blueIFS.getImage()
  );
  
  set(0, 0, compo);
  hist3d.setImage(compo);

}

void RA11(float n) { redMul[0][0] = n; }
void RA12(float n) { redMul[1][0] = n; }
void RA21(float n) { redMul[0][1] = n; }
void RA22(float n) { redMul[1][1] = n; }
void RB11(float n) { redAdd[0][0] = n; }
void RB12(float n) { redAdd[1][0] = n; }
void RB21(float n) { redAdd[0][1] = n; }
void RB22(float n) { redAdd[1][1] = n; }

void GA11(float n) { greenMul[0][0] = n; }
void GA12(float n) { greenMul[1][0] = n; }
void GA21(float n) { greenMul[0][1] = n; }
void GA22(float n) { greenMul[1][1] = n; }
void GB11(float n) { greenAdd[0][0] = n; }
void GB12(float n) { greenAdd[1][0] = n; }
void GB21(float n) { greenAdd[0][1] = n; }
void GB22(float n) { greenAdd[1][1] = n; }

void BA11(float n) { blueMul[0][0] = n; }
void BA12(float n) { blueMul[1][0] = n; }
void BA21(float n) { blueMul[0][1] = n; }
void BA22(float n) { blueMul[1][1] = n; }
void BB11(float n) { blueAdd[0][0] = n; }
void BB12(float n) { blueAdd[1][0] = n; }
void BB21(float n) { blueAdd[0][1] = n; }
void BB22(float n) { blueAdd[1][1] = n; }

void randomize() {
  Random r = new Random();
  for(Slider s :
    new Slider[] { ra11, ra12, ra21, ra22, 
      ga11, ga12, ga21, ga22, 
      ba11, ba12, ba21, ba22}) {
          s.setValue((float)r.nextGaussian()*5);
      }
  for(Slider s :
    new Slider[] { rb11, rb12, rb21, rb22,
      gb11, gb12, gb21, gb22,
      bb11, bb12, bb21, bb22}) {
          s.setValue((float)r.nextGaussian()*2);
      }
}

void empty() {
  for(Slider s :
    new Slider[] { ra11, ra12, ra21, ra22, rb11, rb12, rb21, rb22,
      ga11, ga12, ga21, ga22, gb11, gb12, gb21, gb22,
      ba11, ba12, ba21, ba22, bb11, bb12, bb21, bb22}) {
        s.setValue(0);
      }
}

private String date() {
    String s = Integer.toString(10000*year()+100*month()+day())
    + Integer.toString(10000*hour()+100*minute()+second())
      + "-" + Integer.toString(millis());
      return s;
}

void export() {
  println("export PNG");
  compo.save("IFS-" + date() + ".png");
}

void rotate() {
  hist3d.setAutorotate(!hist3d.getAutorotate());
}

PImage compose(PImage red, PImage green, PImage blue) {
  PImage tmp = createImage(red.width, red.height, RGB);
  for(int i = 0; i < red.pixels.length; i++) {
    tmp.pixels[i] = color(
      red(red.pixels[i]),
      green(green.pixels[i]),
      blue(blue.pixels[i])
    );
  }
  return tmp;
}

void keyPressed() {
  if(key==' ') {
    hist3d.setAutorotate(!hist3d.getAutorotate());
  }
}
